package com.idiot.operationbackend.controller;

import com.idiot.operationbackend.config.PlatformProperties;
import com.idiot.operationbackend.service.facade.WeChatService;
import com.idiot.operationbackend.support.Constants;
import com.idiot.operationbackend.support.JsonResult;
import com.idiot.operationbackend.support.wechat.WXBizMsgCrypt;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @author wang xiao
 * @date Created in 10:38 2020/9/11
 */
@RestController
@RequestMapping("/wechat")
@Api(value = "WeChatController", tags ="微信")
public class WeChatController {

    private final Logger logger = LoggerFactory.getLogger(WeChatController.class);


    @Autowired
    private WeChatService weChatService;

    @Autowired
    private PlatformProperties properties;



    @GetMapping("preAuth")
    @ApiOperation(value = "获取预授权码地址，微信认证公众号")
    public ResponseEntity<JsonResult<String>> getPreAuthUrl ()   {
        return null;
    }




    @PostMapping("/authCallBack")
    @ApiOperation(value = "第三方平台,授权事件接受url")
    public String componentCallBack(HttpServletRequest request)  {

        String nonce = request.getParameter("nonce");
        String timestamp = request.getParameter("timestamp");
        // 默认aes
        String encryptType = request.getParameter("encrypt_type");
        String msgSignature = request.getParameter("msg_signature");
        logger.info("微信授权事件回调,回调param参数依次是nonce:{},timestamp:{},encrypt_type:{},msg_signature:{}",nonce,
                timestamp,encryptType,msgSignature);

        try {
            WXBizMsgCrypt pc = new WXBizMsgCrypt(properties.getToken(),properties.getSecret(),properties.getAppId());
            BufferedInputStream bis = new BufferedInputStream(request.getInputStream());
            ByteArrayOutputStream buf = new ByteArrayOutputStream();
            int result = bis.read();
            while(result != -1) {
                buf.write((byte) result);
                result = bis.read();
            }
            String postData = buf.toString("utf-8");
            logger.info("微信授权事件回调,postData before decry :{}",postData);
            postData = pc.decryptMsg(msgSignature,timestamp,nonce,postData);
            logger.info("微信授权事件回调,postData after decry :{}",postData);
            return weChatService.authCallBack(xmlToMap(postData));
        }catch (Exception e) {
            logger.info("微信授权事件回调 ------->失败,message is :{}",e.getMessage());
            return Constants.SUCCESS;
        }
    }



    /**
     *  微信消息转换成map(非加密)
     * @author wangxiao
     * @date 14:10 2020/9/11
     * @param inputStream
     * @return java.util.Map<java.lang.String,java.lang.String>
     */
    private Map<String, String> xmlToMap(InputStream inputStream) {
        SAXReader saxReader = new SAXReader();
        try {
            Document document = saxReader.read(inputStream);
            return parseDocument(document);
        }catch (Exception e) {
            logger.error("xml 转换出错");
            return null;
        }
    }


    /**
     *  微信消息转换成map(非加密)
     * @author wangxiao
     * @date 14:10 2020/9/11
     * @param data
     * @return java.util.Map<java.lang.String,java.lang.String>
     */
    private Map<String, String> xmlToMap(String data) {
        SAXReader saxReader = new SAXReader();
        try {
            Document document = saxReader.read(data);
            return parseDocument(document);
        }catch (Exception e) {
            logger.error("xml 转换出错");
            return null;
        }
    }

    private Map<String,String> parseDocument (Document document) {
        Map<String, String> map = new HashMap<>(10);
        Element root = document.getRootElement();
        List<Element> elements = root.elements();
        for (Element element : elements) {
            map.put(element.getName(), element.getStringValue());
        }
        return map;
    }
}
